﻿


# Function to authenticate to BitTitan SDK
Function Connect-BitTitan {
    [CmdletBinding()]
    # Authenticate
    $script:creds = Get-Credential -Message "Enter BitTitan credentials"

    if(!$script:creds) {
        $msg = "ERROR: Failed to authenticate with BitTitan. Please enter valid BitTitan Credentials. Script aborted."
        Write-Host -ForegroundColor Red  $msg
        Log-Write -Message $msg
        Exit
    }
    try { 
        # Get a ticket and set it as default
        $script:ticket = Get-BT_Ticket -Credentials $script:creds -SetDefault -ServiceType BitTitan -ErrorAction SilentlyContinue
        # Get a MW ticket
        $script:mwTicket = Get-MW_Ticket -Credentials $script:creds -ErrorAction SilentlyContinue 
    }
    catch {

        $currentPath = Split-Path -parent $script:MyInvocation.MyCommand.Definition
        $moduleLocations = @("$currentPath\BitTitanPowerShell.dll", "$env:ProgramFiles\BitTitan\BitTitan PowerShell\BitTitanPowerShell.dll",  "${env:ProgramFiles(x86)}\BitTitan\BitTitan PowerShell\BitTitanPowerShell.dll")
        foreach ($moduleLocation in $moduleLocations) {
            if (Test-Path $moduleLocation) {
                Import-Module -Name $moduleLocation

                # Get a ticket and set it as default
                $script:ticket = Get-BT_Ticket -Credentials $script:creds -SetDefault -ServiceType BitTitan -ErrorAction SilentlyContinue
                # Get a MW ticket
                $script:mwTicket = Get-MW_Ticket -Credentials $script:creds -ErrorAction SilentlyContinue 

                if(!$script:ticket -or !$script:mwTicket) {
                    $msg = "ERROR: Failed to authenticate with BitTitan. Please enter valid BitTitan Credentials. Script aborted."
                    Write-Host -ForegroundColor Red  $msg
                    Log-Write -Message $msg
                    Exit
                }
                else {
                    $msg = "SUCCESS: Connected to BitTitan."
                    Write-Host -ForegroundColor Green  $msg
                    Log-Write -Message $msg
                }

                return
            }
        }

        $msg = "ACTION: Install BitTitan PowerShell SDK 'bittitanpowershellsetup.msi' downloaded from 'https://www.bittitan.com' and execute the script from there."
        Write-Host -ForegroundColor Yellow $msg
        Write-Host

        Sleep 5

        $url = "https://www.bittitan.com/downloads/bittitanpowershellsetup.msi " 
        $result= Start-Process $url

        Exit
    }  

    if(!$script:ticket -or !$script:mwTicket) {
        $msg = "ERROR: Failed to authenticate with BitTitan. Please enter valid BitTitan Credentials. Script aborted."
        Write-Host -ForegroundColor Red  $msg
        Log-Write -Message $msg
        Exit
    }
    else {
        $msg = "SUCCESS: Connected to BitTitan."
        Write-Host -ForegroundColor Green  $msg
        Log-Write -Message $msg
    }
}


Function Connect-Office365 {
    [CmdletBinding()]
    # Authenticate
    $script:o365creds = Get-Credential -Message "Enter Office 365 Tenant credentials"

    if(!$script:creds) {
        $msg = "ERROR: Failed to authenticate with Office 365. Please enter valid Office 365 Credentials. Script aborted."
        Write-Host -ForegroundColor Red  $msg
        Log-Write -Message $msg
        Exit
    }
    try { 

        Connect-ExchangeOnline -Credential $script:o365creds -ShowProgress $true
        
        #If the login for Office 365 requires MFA then replace the Credential Option above with the following
        #Connect-ExchangeOnline -UserPrincipalName <UPN> -ShowProgress $true

        import-module msonline
        connect-MsolService -Credential $script:o365creds 
        import-module azureadpreview
        connect-AzureAD -Credential $script:o365creds


    }

    catch {
 
        $msg = "FAILED: Unable to connect to Office 365 - Check Credentials and that required PowerShell Modules are available - Exiting"
        Write-Host -ForegroundColor Red $msg
        Log-Write -Message $msg
        Exit
    }  

        $msg = "SUCCESS: Connected to Office 365."
        Write-Host -ForegroundColor Green  $msg
        Log-Write -Message $msg

    }



function Import-MigrationWizModule {
    if (((Get-Module -Name "BitTitanPowerShell") -ne $null) -or ((Get-InstalledModule -Name "BitTitanManagement" -ErrorAction SilentlyContinue) -ne $null)) {
        return
    }

    $currentPath = Split-Path -parent $script:MyInvocation.MyCommand.Definition
    $moduleLocations = @("$currentPath\BitTitanPowerShell.dll", "$env:ProgramFiles\BitTitan\BitTitan PowerShell\BitTitanPowerShell.dll",  "${env:ProgramFiles(x86)}\BitTitan\BitTitan PowerShell\BitTitanPowerShell.dll")
    foreach ($moduleLocation in $moduleLocations) {
        if (Test-Path $moduleLocation) {
            Import-Module -Name $moduleLocation
            return
        }
    }
    
    $msg = "INFO: BitTitan PowerShell SDK not installed."
    Write-Host -ForegroundColor Red $msg 

    Write-Host
    $msg = "ACTION: Install BitTitan PowerShell SDK 'bittitanpowershellsetup.msi' downloaded from 'https://www.bittitan.com'."
    Write-Host -ForegroundColor Yellow $msg

    Sleep 5

    $url = "https://www.bittitan.com/downloads/bittitanpowershellsetup.msi " 
    $result= Start-Process $url
    Exit

}

# Function to display the workgroups created by the user
Function Select-MSPC_Workgroup {

    #######################################
    # Display all mailbox workgroups
    #######################################

    $workgroupPageSize = 100
  	$workgroupOffSet = 0
	$workgroups = @()

    Write-Host
    Write-Host -Object  "INFO: Retrieving MSPC workgroups..."

   do {
       try {
            #default workgroup in the 1st position
            $workgroupsPage = @(Get-BT_Workgroup -PageOffset $workgroupOffset -PageSize 1 -IsDeleted false -CreatedBySystemUserId $script:ticket.SystemUserId )
        }
        catch {
            $msg = "ERROR: Failed to retrieve MSPC workgroups."
            Write-Host -ForegroundColor Red  $msg
            Log-Write -Message $msg
            Write-Host -ForegroundColor Red $_.Exception.Message
            Log-Write -Message $_.Exception.Message
            Exit
        }

        if($workgroupsPage) {
            $workgroups += @($workgroupsPage)
        }

        $workgroupOffset += 1
    } while($workgroupsPage)

    $workgroupOffSet = 0

    do { 
        try{
            #add all the workgroups including the default workgroup, so there will be 2 default workgroups
            $workgroupsPage = @(Get-BT_Workgroup -PageOffset $workgroupOffSet -PageSize $workgroupPageSize -IsDeleted false | where { $_.CreatedBySystemUserId -ne $script:ticket.SystemUserId })   
        }
        catch {
            $msg = "ERROR: Failed to retrieve MSPC workgroups."
            Write-Host -ForegroundColor Red  $msg
            Log-Write -Message $msg
            Write-Host -ForegroundColor Red $_.Exception.Message
            Log-Write -Message $_.Exception.Message
            Exit
        }
        if($workgroupsPage) {
            $workgroups += @($workgroupsPage)
            foreach($Workgroup in $workgroupsPage) {
                Write-Progress -Activity ("Retrieving workgroups (" + $($workgroups.Length -1) + ")") -Status $Workgroup.Id
            }

            $workgroupOffset += $workgroupPageSize
        }
    } while($workgroupsPage)

    if($workgroups -ne $null -and $workgroups.Length -ge 1) {
        Write-Host -ForegroundColor Green -Object ("SUCCESS: "+ $($workgroups.Length -1).ToString() + " Workgroup(s) found.") 
    }
    else {
        Write-Host -ForegroundColor Red -Object  "INFO: No workgroups found." 
        Exit
    }

    #######################################
    # Prompt for the mailbox Workgroup
    #######################################
    if($workgroups -ne $null)
    {
        Write-Host -ForegroundColor Yellow -Object "ACTION: Select a Workgroup:" 
        Write-Host -Object "INFO: A default workgroup has no name, only Id. Your default workgroup is the number 0 in yellow." 

        for ($i=0; $i -lt $workgroups.Length; $i++) {
            
            $Workgroup = $workgroups[$i]

            if([string]::IsNullOrEmpty($Workgroup.Name)) {
                if($i -eq 0) {
                    $defaultWorkgroupId = $Workgroup.Id.Guid
                    Write-Host -ForegroundColor Yellow -Object $i,"-",$defaultWorkgroupId
                }
                else {
                    if($Workgroup.Id -ne $defaultWorkgroupId) {
                        Write-Host -Object $i,"-",$Workgroup.Id
                    }
                }
            }
            else {
                Write-Host -Object $i,"-",$Workgroup.Name
            }
        }
        Write-Host -Object "x - Exit"
        Write-Host

        do {
            if($workgroups.count -eq 1) {
                $msg = "INFO: There is only one workgroup. Selected by default."
                Write-Host -ForegroundColor yellow  $msg
                Log-Write -Message $msg
                $Workgroup=$workgroups[0]
                Return $Workgroup.Id
            }
            else {
                $result = Read-Host -Prompt ("Select 0-" + ($workgroups.Length-1) + ", or x")
            }
            
            if($result -eq "x")
            {
                Exit
            }
            if(($result -match "^\d+$") -and ([int]$result -ge 0) -and ([int]$result -lt $workgroups.Length))
            {
                $Workgroup=$workgroups[$result]
                Return $Workgroup.Id
            }
        }
        while($true)

    }

}

### Function to display all customers
Function Select-MSPC_Customer {

    param 
    (      
        [parameter(Mandatory=$true)] [String]$WorkgroupId
    )

    #######################################
    # Display all mailbox customers
    #######################################

    $customerPageSize = 100
  	$customerOffSet = 0
	$customers = $null

    Write-Host
    Write-Host -Object  "INFO: Retrieving MSPC customers..."

    do
    {   
        try { 
            $customersPage = @(Get-BT_Customer -WorkgroupId $WorkgroupId -IsDeleted False -IsArchived False -PageOffset $customerOffSet -PageSize $customerPageSize)
        }
        catch {
            $msg = "ERROR: Failed to retrieve MSPC customers."
            Write-Host -ForegroundColor Red  $msg
            Log-Write -Message $msg
            Write-Host -ForegroundColor Red $_.Exception.Message
            Log-Write -Message $_.Exception.Message
            Exit
        }
    
        if($customersPage) {
            $customers += @($customersPage)
            foreach($customer in $customersPage) {
                #Write-Progress -Activity ("Retrieving customers (" + $customers.Length + ")") -Status $customer.CompanyName
            }
            
            $customerOffset += $customerPageSize
        }

    } while($customersPage)

    

    if($customers -ne $null -and $customers.Length -ge 1) {
        Write-Host -ForegroundColor Green -Object ("SUCCESS: "+ $customers.Length.ToString() + " customer(s) found.") 
    }
    else {
        Write-Host -ForegroundColor Red -Object  "INFO: No customers found." 
        Exit
    }

    #######################################
    # {Prompt for the mailbox customer
    #######################################
    if($customers -ne $null)
    {
        Write-Host -ForegroundColor Yellow -Object "ACTION: Select a customer:" 

        for ($i=0; $i -lt $customers.Length; $i++)
        {
            $customer = $customers[$i]
            Write-Host -Object $i,"-",$customer.CompanyName
        }
        Write-Host -Object "x - Exit"
        Write-Host

        do
        {
            if($customers.count -eq 1) {
                $msg = "INFO: There is only one customer. Selected by default."
                Write-Host -ForegroundColor yellow  $msg
                Log-Write -Message $msg
                $customer=$customers[0]

                try{
                    if($script:confirmImpersonation -ne $null -and $script:confirmImpersonation.ToLower() -eq "y") {
                        $script:customerTicket = Get-BT_Ticket -Credentials $script:creds -OrganizationId $Customer.OrganizationId -ImpersonateId $script:mspcSystemUserId -ErrorAction Stop
                    }
                    else{
                        $script:customerTicket = Get-BT_Ticket -Credentials $script:creds -OrganizationId $Customer.OrganizationId -ErrorAction Stop
                    }
                }
                Catch{
                    Write-Host -ForegroundColor Red "ERROR: Cannot create the customer ticket." 
                }

                $script:customerName = $Customer.CompanyName

                Return $customer
            }
            else {
                $result = Read-Host -Prompt ("Select 0-" + ($customers.Length-1) + ", or x")
            }

            if($result -eq "x")
            {
                Exit
            }
            if(($result -match "^\d+$") -and ([int]$result -ge 0) -and ([int]$result -lt $customers.Length))
            {
                $customer=$customers[$result]
                                
                try{
                    if($script:confirmImpersonation -ne $null -and $script:confirmImpersonation.ToLower() -eq "y") {
                        $script:customerTicket = Get-BT_Ticket -Credentials $script:creds -OrganizationId $Customer.OrganizationId -ImpersonateId $script:mspcSystemUserId -ErrorAction Stop
                    }
                    else{
                        $script:customerTicket = Get-BT_Ticket -Credentials $script:creds -OrganizationId $Customer.OrganizationId -ErrorAction Stop
                    }
                }
                Catch{
                    Write-Host -ForegroundColor Red "ERROR: Cannot create the customer ticket." 
                }

                $script:customerName = $Customer.CompanyName

                Return $Customer
            }
        }
        while($true)

    }

}

### Function to display all mailbox connectors
Function Select-MW_Connector {

    param 
    (      
        [parameter(Mandatory=$true)] [guid]$customerOrganizationId
    )

        write-host 
$msg = "####################################################################################################`
                       SELECT CONNECTOR(S)              `
####################################################################################################"
Write-Host $msg
    
    #######################################
    # Display all mailbox connectors
    #######################################
    $connectorOffSet = 0
    $connectorPageSize = 100
    $mailboxPageSize = 100
    $script:connectors = $null

    Write-Host
    Write-Host -Object  "Retrieving all connectors ..."

    do {
        $connectorsPage = @(Get-MW_MailboxConnector -Ticket $mwTicket -OrganizationId $customerOrgId -PageOffset $connectorOffSet -PageSize $connectorPageSize | sort ProjectType,Name )
    
        if($connectorsPage) {
            $script:connectors += @($connectorsPage)
            foreach($connector in $connectorsPage) {
                Write-Progress -Activity ("Retrieving connectors (" + $script:connectors.Length + ")") -Status $connector.Name
            }

            $connectorOffset += $connectorPageSize
        }

    } while($connectorsPage)

    if($script:connectors -ne $null -and $script:connectors.Length -ge 1) {
        Write-Host -ForegroundColor Green -Object ("SUCCESS: "+ $script:connectors.Length.ToString() + " connector(s) found.") 
    }
    else {
        Write-Host -ForegroundColor Red -Object  "INFO: No connectors found." 
        Exit
    }

    #######################################
    # {Prompt for the mailbox connector
    #######################################
    $allConnectors = $false
    if($connectors -ne $null) {
        Write-Host -ForegroundColor Yellow -Object "Select a connector:" 

        for ($i=0; $i -lt $script:connectors.Length; $i++) {
            $connector = $script:connectors[$i]
            Write-Host -Object $i,"-",$connector.Name,"-",$connector.ProjectType
        }
        Write-Host -ForegroundColor Yellow  -Object "C - Select project names from CSV file"
        Write-Host -ForegroundColor Yellow  -Object "A - Export all project statitics"
        Write-Host -Object "x - Exit"
        Write-Host

        do {
            $result = Read-Host -Prompt ("Select 0-" + ($script:connectors.Length-1) + ", A or x")
            if($result -eq "x") {
                Exit
            }
            if($result -eq "C") {
                $script:ProjectsFromCSV = $true
                $script:allConnectors = $false

                $script:selectedConnectors = @()

                Write-Host -ForegroundColor yellow "ACTION: Select the CSV file to import project names."

                $workingDir = "C:\scripts"
                $result = Get-FileName $workingDir

                #Read CSV file
                try {
                    $projectsInCSV = @((import-CSV $script:inputFile | Select ProjectName -unique).ProjectName)                    
                    if(!$projectsInCSV) {$projectsInCSV = @(get-content $script:inputFile | where {$_ -ne "ProjectName"})}
                    Write-Host -ForegroundColor Green "SUCCESS: $($projectsInCSV.Length) projects imported." 

                    :AllConnectorsLoop
                    foreach($connector in $script:connectors) {  

                        $notFound = $false

                        foreach ($projectInCSV in $projectsInCSV) {
                            if($projectInCSV -eq $connector.Name) {
                                $notFound = $false
                                Break
                            } 
                            else {                               
                                $notFound = $true
                            } 
                        }

                        if($notFound) {
                            Continue AllConnectorsLoop
                        }  
                        
                        $script:selectedConnectors += $connector                                           
                    }	
                }
                catch {
                    $msg = "ERROR: Failed to import the CSV file '$script:inputFile'. All projects will be processed."
                    Write-Host -ForegroundColor Red  $msg
                    Log-Write -Message $msg 
                    Log-Write -Message $_.Exception.Message

                    $script:allConnectors = $True
                    $script:ProjectsFromCSV = $false
                }          
                
                Break
            }
            if($result -eq "A") {
                $script:ProjectsFromCSV = $false
                $script:allConnectors = $true

                Break
            }
            if(($result -match "^\d+$") -and ([int]$result -ge 0) -and ([int]$result -lt $script:connectors.Length)) {
                $script:ProjectsFromCSV = $false
                $script:allConnectors = $false

                $script:connector=$script:connectors[$result]
                Break
            }
        } while($true)

write-host 
$msg = "####################################################################################################`
                       EXPORT (ALL) MIGRATIONWIZ PROJECT STATISTICS              `
####################################################################################################"
Write-Host $msg        
    
        if($script:allConnectors -or $script:ProjectsFromCSV) {

            $currentConnector = 0

            if($script:ProjectsFromCSV -and !$script:allConnectors) {
                $allConnectors = $script:selectedConnectors 
                $connectorsCount = $script:selectedConnectors.Count           
            }
            else {
                $allConnectors = $script:connectors
                $connectorsCount = $script:connectors.Count
            }

            foreach($connector in $allConnectors) {
                #######################################
                # Get mailboxes
                #######################################
                $mailboxOffSet = 0
                $mailboxes = $null

                $currentConnector += 1

                Write-Host
                Write-Host -Object  ("Retrieving migration information of $currentConnector/$connectorsCount project '$($connector.Name)':")

                do {
                    $mailboxesPage = @(Get-MW_Mailbox -Ticket $mwTicket -ConnectorId $connector.Id -PageOffset $mailboxOffSet -PageSize $mailboxPageSize)
                    if($mailboxesPage) {
                        $mailboxes += @($mailboxesPage)
                        foreach($mailbox in $mailboxesPage) {
                            if ($connector.Type -eq "Mailbox" -or "Archive") {
                                if (-not ([string]::IsNullOrEmpty($mailbox.ExportEmailAddress))) {
                                    Write-Progress -Activity ("Retrieving mailboxes for " + $connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ExportEmailAddress
                                }
                                elseif (-not ([string]::IsNullOrEmpty($mailbox.ImportEmailAddress))) {
                                    Write-Progress -Activity ("Retrieving mailboxes for " + $connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ImportEmailAddress
                                }
                            }
                            elseif ($connector.Type -eq "Storage") {
                                if (-not ([string]::IsNullOrEmpty($mailbox.ExportEmailAddress))) {
                                    Write-Progress -Activity ("Retrieving document migrations for " + $connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ExportEmailAddress
                                }
                                elseif (-not ([string]::IsNullOrEmpty($mailbox.ImportEmailAddress))) {
                                    Write-Progress -Activity ("Retrieving document migrations for " + $connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ImportEmailAddress
                                }
                                elseif (-not ([string]::IsNullOrEmpty($mailbox.ImportLibrary))) {
                                    Write-Progress -Activity ("Retrieving document libraries for " + $connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ImportLibrary
                                }
                                elseif (-not ([string]::IsNullOrEmpty($mailbox.ExportLibrary))) {
                                    Write-Progress -Activity ("Retrieving document libraries for " + $connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ExportLibrary
                                }
                            }
                        }

                        $mailboxOffSet += $mailboxPageSize
                    }
                } while($mailboxesPage)

                if($mailboxes -ne $null -and $mailboxes.Length -ge 1) {
                    Write-Host -ForegroundColor Green -Object ("SUCCESS: " + $mailboxes.Length.ToString() + " migration(s) found")
                }
                else {
                    Write-Host -ForegroundColor Red -Object  "INFO: no migrations found." 
                    Return
                }
                if ($connector.ProjectType -eq "Storage") {
                    Get-DocumentConnectorStatistics -mailboxes $mailboxes -connectorName $connector.Name
                }
                elseif (($connector.ProjectType -eq "Mailbox" -or $connector.ProjectType -eq "Archive") -and ($connector.ProjectType -ne "Storage")) {
                    Get-MailboxConnectorStatistics -mailboxes $mailboxes -connectorName $connector.Name
                }
                else {
                    Write-Host "The project you selected is from an invalid type. Aborting script"
                    Exit
                }
            }
        }
        else{
            #######################################
            # Get mailboxes
            #######################################
            $mailboxOffSet = 0
            $mailboxes = $null

            Write-Host
            Write-Host -Object  ("Retrieving migrations for project '$($script:connector.Name)':")

            do {
                $mailboxesPage = @(Get-MW_Mailbox -Ticket $mwTicket -ConnectorId $script:connector.Id -PageOffset $mailboxOffSet -PageSize $mailboxPageSize)
                if($mailboxesPage) {
                    $mailboxes += @($mailboxesPage)
                    foreach($mailbox in $mailboxesPage) {
                        if ($script:connector.Type -eq "Mailbox" -or "Archive") {
                            if (-not ([string]::IsNullOrEmpty($mailbox.ExportEmailAddress))) {
                                Write-Progress -Activity ("Retrieving mailboxes for " + $script:connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ExportEmailAddress
                            }
                            elseif (-not ([string]::IsNullOrEmpty($mailbox.ImportEmailAddress))) {
                                Write-Progress -Activity ("Retrieving mailboxes for " + $script:connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ImportEmailAddress
                            }
                        }
                        elseif ($connector.Type -eq "Storage") {
                            if (-not ([string]::IsNullOrEmpty($mailbox.ExportEmailAddress))) {
                                Write-Progress -Activity ("Retrieving document migrations for " + $script:connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ExportEmailAddress
                            }
                            elseif (-not ([string]::IsNullOrEmpty($mailbox.ImportEmailAddress))) {
                                Write-Progress -Activity ("Retrieving document migrations for " + $script:connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ImportEmailAddress
                            }
                            elseif (-not ([string]::IsNullOrEmpty($mailbox.ImportLibrary))) {
                                Write-Progress -Activity ("Retrieving document libraries for " + $script:connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ImportLibrary
                            }
                            elseif (-not ([string]::IsNullOrEmpty($mailbox.ExportLibrary))) {
                                Write-Progress -Activity ("Retrieving document libraries for " + $script:connector.Name + " (" + $mailboxes.Length + ")") -Status $mailbox.ExportLibrary
                            }
                        }
                    }

                    $mailboxOffSet += $mailboxPageSize
                }
            } while($mailboxesPage)

            if($mailboxes -ne $null -and $mailboxes.Length -ge 1) {
                Write-Host -ForegroundColor Green -Object ("SUCCESS: " + $mailboxes.Length.ToString() + " migration(s) found")
            }
            else {
                Write-Host -ForegroundColor Red -Object  "INFO: no migrations found." 
                Return
            }

            if ($script:connector.ProjectType -eq "Storage") {
                Get-DocumentConnectorStatistics -mailboxes $mailboxes -connectorName $script:connector.Name
            }
            elseif (($script:connector.ProjectType -eq "Mailbox" -or $script:connector.ProjectType -eq "Archive") -and ($script:connector.ProjectType -ne "Storage")) {
                Get-MailboxConnectorStatistics -mailboxes $mailboxes -connectorName $script:connector.Name
            }
            else {
                Write-Host "The project you selected is from an invalid type. Aborting script"
                Exit
            }
        }
    } 
}

### Function to create the working and log directories
Function Create-Working-Directory {    
    param 
    (
        [CmdletBinding()]
        [parameter(Mandatory=$true)] [string]$workingDir,
        [parameter(Mandatory=$true)] [string]$logDir
    )
    if ( !(Test-Path -Path $workingDir)) {
		try {
			$suppressOutput = New-Item -ItemType Directory -Path $workingDir -Force -ErrorAction Stop
            $msg = "SUCCESS: Folder '$($workingDir)' for CSV files has been created."
            Write-Host -ForegroundColor Green $msg
		}
		catch {
            $msg = "ERROR: Failed to create '$workingDir'. Script will abort."
            Write-Host -ForegroundColor Red $msg
            Exit
		}
    }
    if ( !(Test-Path -Path $logDir)) {
        try {
            $suppressOutput = New-Item -ItemType Directory -Path $logDir -Force -ErrorAction Stop      

            $msg = "SUCCESS: Folder '$($logDir)' for log files has been created."
            Write-Host -ForegroundColor Green $msg 
        }
        catch {
            $msg = "ERROR: Failed to create log directory '$($logDir)'. Script will abort."
            Write-Host -ForegroundColor Red $msg
            Exit
        } 
    }
}

### Function to write information to the Log File
Function Log-Write {
    param
    (
        [Parameter(Mandatory=$true)]    [string]$Message
    )
    $lineItem = "[$(Get-Date -Format "dd-MMM-yyyy HH:mm:ss") | PID:$($pid) | $($env:username) ] " + $Message
	Add-Content -Path $logFile -Value $lineItem
}


Function Get-FileName($initialDirectory) {
    [System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms") | Out-Null
    
    $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
    $OpenFileDialog.initialDirectory = $initialDirectory
    $OpenFileDialog.filter = "CSV (*.csv)| *.csv"
    $OpenFileDialog.ShowDialog() | Out-Null
    $script:inputFile = $OpenFileDialog.filename

    if($OpenFileDialog.filename -eq "") {
        Return $false
    }
    else{
        $msg = "SUCCESS: CSV file '$($OpenFileDialog.filename)' selected."
        Write-Host -ForegroundColor Green  $msg
        Log-Write -Message $msg
        Return $true
    }
}

Function Get-Directory($initialDirectory) {
    [System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms") | Out-Null    
    $FolderBrowser = New-Object System.Windows.Forms.FolderBrowserDialog
    $FolderBrowser.ShowDialog()| Out-Null

    if($FolderBrowser.SelectedPath -ne "") {
        $workingDir = $FolderBrowser.SelectedPath               
    }
    Write-Host -ForegroundColor Gray  "INFO: Directory '$workingDir' selected."
}


######################################################################################################################################
#                                               MAIN PROGRAM
######################################################################################################################################

Import-MigrationWizModule

#Working Directory
$workingDir = "C:\scripts"

#Logs directory
$logDirName = "Logs"
$logDir = "$workingDir\$logDirName"

#Log file
$logFileName = "$(Get-Date -Format yyyyMMdd-HHmm)_HybridPreFlightChecks.log"
$logFile = "$logDir\$logFileName"

#CSV Report
$CSVFileName = "$(Get-Date -Format yyyyMMdd-HHmm)_HybridPreFlightChecks-Output.csv"
$CSVFile = "$logDir\$CSVFilename"

Create-Working-Directory -workingDir $workingDir -logDir $logDir

$msg = "++++++++++++++++++++++++++++++++++++++++ SCRIPT STARTED ++++++++++++++++++++++++++++++++++++++++"
Log-Write -Message $msg

write-host 
$msg = "####################################################################################################`
                       Connect to Bit Titan                  `
####################################################################################################"
Write-Host $msg
Log-Write -Message "Connecting to your Bit Titan Account"
write-host 

Connect-BitTitan

write-host 
$msg = "####################################################################################################`
                       Connect to Office 365 Tenant             `
####################################################################################################"
Write-Host $msg

Connect-Office365

$msg = "####################################################################################################`
                       Workgroup and Customer Selection             `
####################################################################################################"
Write-Host $msg


#Create the Initial Ticket Request
$ticket = Get-BT_Ticket -Credentials $creds -ServiceType BitTitan -SetDefault

#Select Workgroup
$WorkgroupId = Select-MSPC_WorkGroup

#Select Customer
$customer = Select-MSPC_Customer -Workgroup $WorkgroupId

#Create a ticket for project sharing
$script:mwTicket = Get-MW_Ticket -Credentials $script:creds -WorkgroupId $workgroupId -IncludeSharedProjects 

#Setup the Connector Request
$customerOrgId = $Customer.OrganizationId
$CustomerId = $Customer.Id
$connector = Get-MW_MailboxConnector -ticket $mwticket -OrganizationId $customerorgid

#Reading Tenant Prefix and Calculating Proxy Address

#OldCode for unedited Tenant Name
#$tenantid = Get-AzureADTenantDetail | select displayname
#$tenantidprefix = $tenantid.displayname

$result = Read-Host -Prompt ("Enter the tenant name that preceeds the 'onmicrosoft.com' suffix")
if ($result -ne $null) { $tenantidprefix = $result }

# Get MigrationWiz connector Users
$msg = "Retrieving Hybrid mailboxes"
Log-Write -Message $msg
Write-Progress -Activity "Reading User List from Hybrid Project"
    
try {
    $hybridMailboxes = Get-MW_HybridMailbox -Ticket $mwTicket -ConnectorId $connector.id -RetrieveAll | Sort-Object -Property @{Expression = "ExportEmailAddress"; Descending = $False}
}
catch {
    $msg = "Failed to find Hybrid Mailboxes - Exiting"
    Log-Write -Message $msg
    write-host -foregroundcolor Red $msg
    exit
}

$msg = "Found Mailboxes in Project: Total $($hybridmailboxes.count)"
Log-Write -Message $msg
write-host $msg

$totalmailboxesfound = $hybridMailboxes.count
$currentmailbox=1
$mberrorcount=0

#Check if there are Mailboxes in the Project - If not then exit
if ($totalmailboxesfound -eq 0) {
    $msg = "No Mailboxes found in Project - Exiting"
    Log-Write -Message $msg
    write-host -foregroundcolor Red $msg
    exit
}

$proxyaddress = $("@$($tenantidprefix).mail.onmicrosoft.com")
$msg = "Checking for Proxy Address:  $proxyaddress "
Log-Write -Message $msg
write-host -foregroundcolor Yellow $msg
    
#Prepare Output for CSV File
$output = @()

foreach ($mb in $hybridMailboxes) {

    Write-Progress -Activity "Performing Pre Flight Checks" -Status $mb.primarysmtpaddress -PercentComplete ($currentmailbox/$totalmailboxesfound*100)
    $o365mb = get-azureaduser -objectid $mb.primarysmtpaddress

    $foundproxy=$false

    foreach ($pa in $o365mb.ProxyAddresses) {
           
            if ($pa -like "*$proxyaddress") {
            $foundproxy=$true
            }

        }

			# Add Proxy address flag based on proxy address availability
			if (!($foundproxy)) {
			write-host "Proxy Address is NOT Present - $($o365mb.displayname) - $($mb.primarysmtpaddress)" -ForegroundColor Red

            #Add Object to CSV
            $userObj = New-Object PSObject
            $userObj | Add-Member NoteProperty -Name "Display Name” -Value $o365mb.displayname
            $userObj | Add-Member NoteProperty -Name "SMTPAddress” -Value $mb.PrimarySmtpAddress
            $userObj | Add-Member NoteProperty -Name "Proxy Address Missing” -Value $proxyaddress
            
            $output += $userObj
            $mberrorcount ++

			} else {
			    #Test Code left in for easier customization by client if needed	
                #write-host "Proxy Address is Present" -ForegroundColor Green
			}

    $currentmailbox ++
}


#Check and Report Results

if ($mberrorcount -eq 0 ) {

        write-host "-----------------------------------------------------------------------------------------------"
        $msg = "Script Completed - No Proxy Address Issues Found in Mailboxes"
        Log-Write -Message $msg
        write-host -foregroundcolor Green $msg

        write-host "-----------------------------------------------------------------------------------------------"
        write-host
        write-host

    } else {

        write-host "-----------------------------------------------------------------------------------------------"
        $msg = "Script Completed - Total Mailboxes with Proxy Address Issues: $mberrorcount"
        Log-Write -Message $msg
        write-host -foregroundcolor Yellow $msg

        $msg = "Check the CSV File created for mailbox details"
        Log-Write -Message $msg
        write-host  $msg

        $msg = $CSVFile
        Log-Write -Message $msg
        write-host  $msg
        
        $output |export-csv -Path $CSVFile -NoTypeInformation
        (Get-Content $CSVFile) -replace '(?m)"([^,]*?)"(?=,|$)', '$1' | Set-Content $CSVFile

        write-host "-----------------------------------------------------------------------------------------------"
        write-host
        write-host
    }
